@@ -7,29 +7,27 @@ $| = 1;  # fast pipes
 # Good for making sure the first few M-R bases are set up correctly.
 
 use Math::Prime::Util qw/is_prob_prime/;
-use Math::Prime::Util::PrimeArray;
-
-my @primes;  tie @primes, 'Math::Prime::Util::PrimeArray';
+use Math::Primality qw/next_prime/;
 
 # Test just primes
 if (0) {
-  foreach my $i (1 .. 10000000) {
-    my $n = shift @primes;
-    die unless is_prob_prime($n);
+  my $n = 2;
+  my $i = 1;
+  while ($n < 100_000_000) {
+    die "$n" unless is_prob_prime($n);
+    $n = next_prime($n);  $n = int("$n");
     #print "." unless $i % 16384;
-    print "$i $n\n" unless $i % 262144;
+    print "$i $n\n" unless $i++ % 16384;
   }
 }
 
 # Test every number up to the 100Mth prime (about 2000M)
 if (1) {
-  die "2 should be prime" unless is_prob_prime(2);
-  shift @primes;
-  my $n = shift @primes;
+  my $n = 2;
   foreach my $i (2 .. 100_000_000) {
     die "$n should be prime" unless is_prob_prime($n);
     print "$i $n\n" unless $i % 262144;
-    my $next = shift @primes;
+    my $next = next_prime($n);  $next = int("$next");
     my $diff = ($next - $n) >> 1;
     if ($diff > 1) {
       foreach my $d (1 .. $diff-1) {
