@@ -630,6 +630,108 @@ UV _XS_nth_prime(UV n)
   return ( (segbase*30) + p );
 }
 
+/* Return an IV array with lo-hi+1 elements.  mu[k-lo] = µ(k) for k = lo .. hi.
+ * It is the callers responsibility to call Safefree on the result. */
+IV* _moebius_range(UV lo, UV hi)
+{
+  IV* mu;
+  UV i, p, sqrtn, range;
+
+  /* This implementation follows that of Deléglise & Rivat (1996), which is
+   * a segmented version of Lioen & van de Lune (1994).
+   */
+  range = hi-lo+1;
+  sqrtn = (UV) (sqrt(hi) + 0.5);
+
+  New(0, mu, range, IV);
+  if (mu == 0)
+    croak("Could not get memory for %"UVuf" moebius results\n", range);
+  for (i = lo; i <= hi; i++)
+    mu[i-lo] = 1;
+  if (lo == 0)  mu[0] = 0;
+  prime_precalc(sqrtn);
+  for (p = 2; p <= sqrtn; p = _XS_next_prime(p)) {
+    i = p*p;
+    if (i < lo)
+      i = i*(lo/i) + ( (lo%i) ? i : 0 );
+    while (i <= hi) {
+      mu[i-lo] = 0;
+      i += p*p;
+    }
+    i = p;
+    if (i < lo)
+      i = i*(lo/i) + ( (lo%i) ? i : 0 );
+    while (i <= hi) {
+      mu[i-lo] *= -p;
+      i += p;
+    }
+  }
+  for (i = lo; i <= hi; i++) {
+    IV m = mu[i-lo];
+    if (m != i && m != -i)  m *= -1;
+    mu[i-lo] = (m>0) - (m<0);
+  }
+  return mu;
+}
+
+IV _XS_mertens(UV n) {
+#if 0
+  /* Benito and Varona 2008, theorem 3.  Segment. */
+  IV* mu;
+  UV k;
+  UV limit = 1000000;
+  UV n3 = n/3;
+  IV sum = 0;
+  UV startk = 1;
+  UV endk = limit;
+  prime_precalc( (UV) (sqrt(n)+0.5) );
+  while (startk <= n3) {
+    if (endk > n3) endk = n3;
+    mu = _moebius_range(startk, endk);
+    for (k = startk; k <= endk; k++)
+      if (mu[k-startk] != 0)
+        sum += mu[k-startk] * ((n-k)/(2*k));
+    Safefree(mu);
+    startk = endk+1;
+    endk += limit;
+  }
+  return -sum;
+#else
+  /* Deléglise and Rivat (1996) using u = n^1/2 and unsegmented. */
+  /* Very simple, but they use u = n^1/3 and segment */
+  UV u, i, m, nmk;
+  IV* mu;
+  IV* M;
+  IV sum;
+
+  if (n <= 1)  return n;
+  u = (UV) sqrt(n);
+  mu = _moebius_range(0, u);
+  New(0, M, u+1, IV);
+  M[0] = 0;
+  for (i = 1; i <= u; i++)
+    M[i] = M[i-1] + mu[i];
+  sum = M[u];
+  for (m = 1; m <= u; m++) {
+    if (mu[m] != 0) {
+      IV inner_sum = 0;
+      UV lower = (u/m) + 1;
+      UV last_nmk = n/(m*lower);
+      UV this_k = 0;
+      UV next_k = n/(m*1);
+      for (nmk = 1; nmk <= last_nmk; nmk++) {
+        this_k = next_k;
+        next_k = n/(m*(nmk+1));
+        inner_sum += M[nmk] * (this_k - next_k);
+      }
+      sum -= mu[m] * inner_sum;
+    }
+  }
+  Safefree(M);
+  Safefree(mu);
+  return sum;
+#endif
+}
 
 
 
